package com.huigou.topsun.sap.clientMaterials.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.cache.DictUtil;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.clientMaterials.application.ClientMaterialsApplication;
import com.huigou.topsun.sap.clientMaterials.domain.ClientMaterials;
import com.huigou.topsun.sap.clientMaterials.domain.query.ClientMaterialsQueryRequest;
import com.huigou.topsun.sap.clientMaterials.repository.ClientMaterialsRepository;
import com.huigou.topsun.sap.clientMaterials.vo.ClientMaterialsQueryRequestVo;
import com.huigou.topsun.sap.clientMaterials.vo.ClientMaterialsVo;
import com.huigou.topsun.sap.common.HttpClient;
import com.huigou.topsun.sap.common.domain.SapResult;
import com.huigou.topsun.util.SAPUtils;
import com.huigou.uasp.bmp.common.application.BaseApplication;
import com.huigou.util.ClassHelper;
import com.huigou.util.JSONUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Auther: Lxh
 * @Date: 2024/06/05/11:35
 * @Description:
 */
@Service("clientMaterialsApplication")
public class ClientMaterialsApplicationImpl extends BaseApplication implements ClientMaterialsApplication {
    @Autowired
    private ClientMaterialsRepository clientMaterialsRepository;
    @Autowired
    private HttpClient httpClient;


    @Override
    public Map<String, Object> getClientMaterialsList(ClientMaterialsQueryRequest queryRequest) {
        Map<String, Object> map = new HashMap<>();
        //如果是查询已关单的数据，从bpm获取数据
        if ("1".equals(queryRequest.getClosed())){
            QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "clientMaterials");
            QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
            queryModel.putDictionary("closed", DictUtil.getDictionary("yesorno"));
            map = this.sqlExecutorDao.executeSlicedQuery(queryModel);
        }else {
            map = this.getClientMaterialsListFromSap(queryRequest);
        }
        return map;
    }

    @Override
    public Map<String, Object> getClientMaterialsListFromSap(ClientMaterialsQueryRequest queryRequest) {
        List<ClientMaterials>  sapClientMaterialsList = new ArrayList<>();
        Map<String, Object> dataMap = new HashMap<>();
        ClientMaterialsQueryRequestVo queryRequestVo = new ClientMaterialsQueryRequestVo();
        ClassHelper.copyProperties(queryRequest,queryRequestVo);
        ArrayList<Map<String, Object>> maps = new ArrayList<>();
        String json = null;
        try {
            //http://192.168.3.109:8000/sap/bc/erp_ep/search/sch_planorder?sap-client=300
            json = httpClient.execute(queryRequestVo, "search/sch_planorder");
            Map<String, Object> map = JSONUtil.toMap(json);
            SapResult sapResult = JSONObject.parseObject(JSONUtil.toString(map.get("MSG")), SapResult.class);
            if ("S".equals(sapResult.getTYPE())){
                ObjectMapper objectMapper = SAPUtils.objectMapper();
                //忽略不存在
                objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
                //忽略大小写
                objectMapper.configure(MapperFeature.ACCEPT_CASE_INSENSITIVE_PROPERTIES, true);
                sapClientMaterialsList = objectMapper.readValue(JSONUtil.toString(map.get("RDATA")), new TypeReference<List<ClientMaterials>>() {});
                maps = objectMapper.convertValue(sapClientMaterialsList, new TypeReference<ArrayList<Map<String, Object>>>() {
                });
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        dataMap.put("Rows",maps);
        return dataMap;
    }

    @Override
    public ClientMaterials getClientMaterialsById(String id) {
        return clientMaterialsRepository.findOne(id);
    }

    @Override
    public void deleteFromSap(ClientMaterials clientMaterials) {
        Map<String, Object> map = new HashMap<>();
        map.put("PLANNEDORDER",clientMaterials.getPlnum());
        //http://192.168.3.109:8000/sap/bc/erp_ep/del_planorder?sap-client=300
        String json = null;
        try {
            //http://192.168.3.109:8000/sap/bc/erp_ep/del_planorder?sap-client=300
            json = httpClient.execute(map,"del_planorder");
            Map<String, Object> resultMap = JSONUtil.toMap(json);
            SapResult sapResult = JSONObject.parseObject(JSONUtil.toString(resultMap.get("MSG")), SapResult.class);
            if ("S".equals(sapResult.getTYPE())){
                //删除成功，保存数据
                clientMaterials.setClosed("1");
                clientMaterialsRepository.save(clientMaterials);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
